#! /usr/bin/env perl
#
# tag: ewok install apache config files installation script
#

use Getopt::Std;
use File::Basename;
use File::Path;
use File::Copy;
use File::Compare;
use Config::Directory;
use strict;

my $build_dir = dirname($0);
my $Config = Config::Directory->new("conf/dist", {
    glob => [ qw(APACHE_CONF APACHE_USER EWOK_HOME EMBPERL_VERSION) ],
});

my $dist_dir = "dist";
my @conf = qw(ewok.conf ewok-startup.pl htpasswd.ewok);
for (@conf) {
    die "configuration file '$dist_dir/$_' not found" unless -f "$dist_dir/$_";
}
my $hr = '-' x 78;

# Parse options
my %opts;
getopts('n', \%opts);
my $noop = $opts{n};

print $hr, "\n";
print "Installing ewok apache configuration files\n";
print $hr, "\n";

my $apache_user = (getpwnam($Config->{APACHE_USER}))[2]
  if $Config->{APACHE_USER};

# Make APACHE_CONF dir if doesn't exist
if (! -d $Config->{APACHE_CONF}) {
    print "mkdir $Config->{APACHE_CONF}\n";
    if (! $noop) {
        my @new = mkpath($Config->{APACHE_CONF});
        die "creating configuration directory $Config->{APACHE_CONF} failed: $!"
            unless $new[$#new] eq $Config->{APACHE_CONF};
        chown $apache_user, -1, $Config->{APACHE_CONF}
            if $> == 0 && $apache_user;
    }
}

# Copy files
for (@conf) {
    my $filename = $_;
    my $copy = 1;
    if (-f "$Config->{APACHE_CONF}/$_") {
        if (compare("$dist_dir/$_", "$Config->{APACHE_CONF}/$_") == 0) {
            print "-> $Config->{APACHE_CONF}/$_ identical (skipping)\n";
            $copy = 0;
        }
    }
    print "cp $dist_dir/$_ $Config->{APACHE_CONF}/$filename\n" if $copy;
    if (! $noop) {
        if ($copy) {
            copy("$dist_dir/$_", "$Config->{APACHE_CONF}/$filename");
            chown $apache_user, -1, "$Config->{APACHE_CONF}/$filename"
                if $> == 0 && $apache_user;
        }
        # Create symlink in EWOK_HOME/conf (reset filename to initial)
        $filename = $_;
        if (! -e "$Config->{EWOK_HOME}/conf") {
            mkpath "$Config->{EWOK_HOME}/conf" 
                or die "mkdir '$Config->{EWOK_HOME}/conf' failed: $!";
            chown $apache_user, -1, "$Config->{EWOK_HOME}/conf"
                if $> == 0 && $apache_user;
        }
        my $link = "$Config->{EWOK_HOME}/conf/$filename";
        if (! -e $link) {
            symlink "$Config->{APACHE_CONF}/$filename", $link
                or die "symlink $Config->{APACHE_CONF}/me, $link failed: $!";
            chown $apache_user, -1, $link
                if $> == 0 && $apache_user;
        }
        elsif (! -l $link) {
            print "-> Warning: unable to create $Config->{EWOK_HOME}/conf/$filename\nlink - non-link already exists\n";
        }
    }
}

# Modify httpd.conf, if it exists
my $httpd = "$Config->{APACHE_CONF}/httpd-perl.conf";
$httpd = "$Config->{APACHE_CONF}/httpd.conf" if ! -f $httpd;
if (-f $httpd && -w $httpd) {
    open HTTPD, "<$httpd"
        or die "cannot open '$httpd' for read: $!";
    my $hconf;
    {
        local $/;
        $hconf = <HTTPD>;
    }
    close HTTPD;
    # Add the include for "ewok.conf" unless it's already present
    if ($hconf && $hconf !~ m/^(#\s*)?Include.*\bewok.conf"/m) {
        print "-> adding ewok.conf include directive to $httpd\n";
        if (! $noop) {
            open HTTPD, ">>$httpd"
                or die "cannot open '$httpd' for append $!";
            print HTTPD <<EOD;

# Include ewok configuration directives
Include "conf/ewok.conf"
EOD
            close HTTPD;
        }
    }
    elsif ($1) {
        print "-> Warning: $httpd contains ewok.conf include directive,\n   but it's commented out - you'll need to uncomment it to enable ewok\n";
    }
    else {
        print "-> $httpd already contains ewok.conf include directive\n";
    }
}
else {
    print <<EOD;
$hr
ewok was unable to locate your main apache config file (usually httpd.conf) to
add the appropriate include directive for ewok. Please add the following line
to your apache config file so that apache can find ewok:
    Include "conf/ewok.conf"
EOD
}

# What's next
print <<EOD;
$hr
ewok has successfully installed. You should now restart your apache and
then go to http://localhost/ewok/admin (or whatever) to finish configuring 
your installation.
$hr
EOD
