#! /usr/bin/env perl
#
# tag: ewok install main files installation script
#

use Getopt::Std;
use File::Basename;
use File::Path;
use File::Find;
use File::Copy qw(cp);
use File::Compare;
use Config::Directory;
use strict;

my $build_dir = dirname($0);
my $conf_dir = "conf/dist";
my $Config = Config::Directory->new($conf_dir, { 
    glob => [ qw(EWOK_HOME APACHE_USER) ],
});
my $ewok_home = $Config->{EWOK_HOME};
my $apache_user = (getpwnam($Config->{APACHE_USER}))[2]
  if $Config->{APACHE_USER};

my $hr = '-' x 78;

# Parse options
my %opts;
getopts('n', \%opts);
my $noop = $opts{n};

# Make EWOK_HOME dir if doesn't exist
if (! -d $ewok_home) {
  print "mkdir $ewok_home\n";
  if (! $noop) {
    my @new = mkpath($ewok_home);
    die "creating configuration directory $ewok_home failed: $!"
      unless $new[$#new] eq $ewok_home;
    chown $apache_user, -1, $ewok_home if $> == 0 && $apache_user;
  }
}

# Copy files, using File::Find
print $hr, "\n";
print "Installing main ewok files\n";
print $hr, "\n";
my %ignore_dir = map { $_ => 1 } qw(build dist);
my %ignore_file = map { $_ => 1 } qw(lib/OF lib/OF/TemplateBase.pm Makefile configure);
find(sub {
  if ($File::Find::name ne "..") {   # 'next' SEGVs on 5.6
    my ($name) = ($File::Find::name =~ m!^./(.*)!);
    my ($dir) = ($File::Find::dir =~ m!^./(.*)!);
    my ($initial) = ($name =~ m|^([^/]+)|);
    # Check if we want to install this file
    if (! $ignore_dir{$initial} && ! $ignore_file{$name}) {
      # Do a mkpath for directories
      if (! -e "$ewok_home/$dir" && ! $noop) {
        print "mkdir $ewok_home/$dir\n";
        mkpath "$ewok_home/$dir"
          or die "mkdir '$ewok_home/$dir' failed: $!";
        chown $apache_user, -1, "$ewok_home/$dir" if $> == 0 && $apache_user;
      }
      # Symlink links (only works for relative links!)
      if (-l $_) {
        my $prefix = $ewok_home;
        $prefix .= "/$dir" if $dir;
        my $target = readlink($_);
        # Omit absolute links
        if ($target !~ m!^\s*/!) {
          $target = "$prefix/$target";
          1 while ($target =~ s!\w+/../!!);
          print "ln -s $target $prefix/$_\n";
          if (! $noop) {
            unlink "$prefix/$_" if -e "$prefix/$_";
            symlink($target, "$prefix/$_")
              or die "symlink $target, $prefix/$_ failed: $!";
            chown $apache_user, -1, "$prefix/$_" if $> == 0 && $apache_user;
          }
        }
      }
      # Copy files
      elsif (! -d $_) {
        print "cp $name $ewok_home/$dir\n";
        if (! $noop) {
          cp $_, "$ewok_home/$dir"
            or die "cp $name '$ewok_home/$dir' failed: $!";
          chown $apache_user, -1, "$ewok_home/$dir/$_" if $> == 0 && $apache_user;
        }
      }
      # Create directories
      elsif (! -e "$ewok_home/$dir/$_") {
        mkpath "$ewok_home/$dir/$_";
        chown $apache_user, -1, "$ewok_home/$dir/$_" if $> == 0 && $apache_user;
      }
    }
  }
}, ".");

# Post-install steps
# Generate ewok/admin .htaccess file, if doesn't exist
$ENV{EWOK_HOME} = $ewok_home;
unshift @INC, "$ewok_home/lib";
require Ewok::Authz;
if (! -f "$ewok_home/htdocs/admin/.htaccess") {
  Ewok::Authz::refresh_htaccess("$ewok_home/htdocs/admin");
  chown $apache_user, -1, "$ewok_home/htdocs/admin/.htaccess" 
    if $> == 0 && $apache_user;
}
# Initialise metadata.map, if doesn't exist
if (! -f "$ewok_home/htdocs/metadata.map") {
  cp "$ewok_home/htdocs/metadata.map.dist", "$ewok_home/htdocs/metadata.map";
  chown $apache_user, -1, "$ewok_home/htdocs/metadata.map" 
    if $> == 0 && $apache_user;
}
# Initialise sites.map, if doesn't exist
if (! -f "$ewok_home/htdocs/sites.map") {
  cp "$ewok_home/htdocs/sites.map.dist", "$ewok_home/htdocs/sites.map";
  chown $apache_user, -1, "$ewok_home/htdocs/sites.map" 
    if $> == 0 && $apache_user;
}

