#! /usr/bin/env perl
#
# tag: ewok install config munging script
#

use Getopt::Std;
use File::Basename;
use Config::Directory;
use lib './lib';
use OF::TemplateBase;
use strict;

my $build_dir = dirname($0);
my $conf_dir = "conf/dist";

# Parse options
my %opts;
getopts('i:o:s', \%opts);
my $infile = $opts{i};
my $outfile = $opts{o};
die "infile '$infile' not found" if ! -f $infile;

# Load the config settings
die "Running from invalid location - cannot locate $conf_dir directory"
    unless -d $conf_dir;
die "EWOK_HOME not set - have you run 'configure'?" 
    unless -s "$conf_dir/EWOK_HOME";
my $Config = Config::Directory->new($conf_dir, { 
    glob => [ 'APACHE_CONF', 'EWOK_HOME', 'EMBPERL_VERSION' ],
});

my $map = { 
    APACHE_CONF => $Config->{APACHE_CONF}, 
    EWOK_HOME => $Config->{EWOK_HOME}, 
    EMBPERL => ($Config->{EMBPERL_VERSION} == 2 ? 'Embperl' : 'HTML::Embperl'),
    APACHE => ($Config->{EMBPERL_VERSION} == 2 ? 'Apache2' : 'Apache'),
};

# Do the munging
my $t = OF::TemplateBase->new(map => $map, open => '<', close => '>');
$t->munge(infile => $infile, outfile => $outfile);

